/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sf.jannot.alignment.mfa.AlignmentAnnotation;
import net.sf.jannot.shortread.ShortRead;

/**
 * Interface for annotation associated with an EntrySet of type T.
 * 
 * @author Thomas Abeel
 * 
 * @param <T>
 *            type of the annotation
 * @see FeatureAnnotation
 * @see AlignmentAnnotation
 * @see SyntenicAnnotation
 * @see GraphAnnotation
 */
public abstract class EntrySetAnnotation<T> implements Iterable<T> {

	
	/**
	 * Returns all object of type T that overlap with the provided location.
	 * 
	 * @param l
	 * @return
	 */
	public Iterable<T> get(Entry e,Location l){
		return getAll(e);
	}
	
	public abstract boolean contains(T t);
	/**
	 * Returns all object of type T that overlap with the provided location.
	 * 
	 * @param l
	 * @return
	 */
	public Iterable<T> get(Entry e,Location l,int limit){
		return get(e,l);
	}
	
//	public void addAll(EntrySetAnnotation<T> ts){
//		addAll(ts.getAll(e))
//	}
//	
//	public void addAll(Iterable<T> list){
//		for(T t:list)
//			this.list.add(t);
//		setChanged();
//		notifyObservers();
//	}

	

	
	public abstract Iterable<T> getAll(Entry e) ;

	
	public abstract void add(T g);



	@Override
	public abstract Iterator<T> iterator();

	

}
