/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 
 * I must improve this documentation
 * 
 * Class 'Type' contains list of group names which features can be classified.
 * e.g. CDS, intergenic.
 * 
 * Constructor : Type (string name);
 * 
 * Methods: get, toString, moveUp, moveDown
 * 
 * Obs: All previous declared Type names are uniquely stored in a static Type
 * array variable.
 * 
 * @author Thomas Abeel
 * 
 */

public class Type implements DataKey {

	private static ConcurrentHashMap<String, Type> map = new ConcurrentHashMap<String, Type>();

	private static List<Type> order = new ArrayList<Type>();
	static {

	}

	private String name;

	/**
	 * 
	 * Return array of declared types.
	 * 
	 * @return
	 */
	public static Type[] values() {
		return order.toArray(new Type[0]);

	}

	/**
	 * This method is deprecated. Use get(String key) instead.
	 * 
	 */
	@Deprecated
	public static Type valueOf(String s) {
		return get(s);
	}

	/**
	 * 
	 * Return key
	 * 
	 * @param key
	 * @return
	 */
	public synchronized static Type get(String key) {
		if (!map.containsKey(key)) {
			map.put(key, new Type(key));
			order.add(map.get(key));
		}
		return map.get(key);
	}

	private Type(String string) {
		this.name = string;

	}

	/**
 *     
 */
	public String toString() {
		return name;
	}

	/**
	 * Class Type stores all Types defined in a array. moveUp swap 'type' index
	 * with its previous one. If 'type' is the first index, nothing happens.
	 * 
	 * @param type
	 */
	public synchronized static void moveUp(Type type) {
		int index = order.indexOf(type);
		if (index > 0)
			Collections.swap(order, index - 1, index);

	}

	/**
	 * Class Type stores all Types defined in a array. moveDown swap 'type'
	 * index with its next one. If 'type' is the last index, nothing happens.
	 * 
	 * @param type
	 */
	public synchronized static void moveDown(Type type) {
		int index = order.indexOf(type);
		if (index < order.size() - 1)
			Collections.swap(order, index, index + 1);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(DataKey o) {
		return o.toString().compareTo(this.toString());
	}
}
