/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.alignment.maf;

import net.sf.jannot.Strand;
import net.sf.jannot.refseq.Sequence;

/**
 * @author Thomas Abeel
 * 
 */
public abstract class AbstractAlignmentSequence {

	/**
	 * @param id2
	 * @param start
	 *            the real start, not some weird UCSC start, this is ZERO based
	 * @param noNucleotides2
	 * @param srcSize
	 * @param fromSymbol
	 */
	public AbstractAlignmentSequence(String id, int start, int noNucleotides, Strand fromSymbol) {
		this.id = id;
		this.start = start;
		this.noNucleotides = noNucleotides;
		this.strand = fromSymbol;
	}

	protected String id;

	@Override
	public String toString() {
		return id + " " + start() + " " + end();
	}

	// private Entry ref;
	protected int start;
	protected int noNucleotides;
	protected Strand strand;

	public abstract Sequence seq();

	/**
	 * @return
	 */
	public int end() {
		return start + noNucleotides + 1;
	}

	/**
	 * One based coordinate of this multiple alignment block
	 * 
	 * @return
	 */
	public int start() {
		return start + 1;
	}

	public String getName() {
		return id;
	}

	// /**
	// * @return
	// */
	// public Entry entry() {
	// return ref;
	// }

	/**
	 * @return
	 */
	public Strand strand() {
		return strand;
	}

}
