/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.refseq;

/**
 * @author Thomas Abeel
 * 
 */
public class SubSequence extends Sequence {

	private int end;
	private int start;
	private Sequence seq;

	/**
	 * @param start
	 * @param end
	 */
	public SubSequence(Sequence s, int start, int end) {
		this.seq = s;
		this.start = start;
		this.end = end;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.refseq.Sequence#get(int, int)
	 */
	@Override
	public Iterable<Character> get(int start, int end) {
		return seq.get(start + this.start, end);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.refseq.Sequence#get()
	 */
	@Override
	public Iterable<Character> get() {
		return seq.get(this.start, this.end);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.refseq.Sequence#size()
	 */
	@Override
	public int size() {
		return end-start;
	}

}
